using System.Diagnostics;

namespace ComparablePoint
{
    public partial class MainForm : Form
    {
        const int NB_POINTS = 40;
        const int WIN_SIZE = 650;
        const int POINT_SIZE = 9;
        const int OFFSET = 20;
        private List<PointXY> points;

        public MainForm()
        {
            InitializeComponent();

            ClientSize = new Size(WIN_SIZE, WIN_SIZE);
            MaximumSize = MinimumSize = Size;

            points = new List<PointXY>();

            Load += MainForm_Load;
            Paint += MainForm_Paint;
        }

        private void MainForm_Paint(object? sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            Font font = new Font(new FontFamily("Courier New"), 10, FontStyle.Bold, GraphicsUnit.Pixel);
            Pen pen = new Pen(Brushes.LightGray);

            // Dessin des points et leur numro
            for (int i = 0; i < NB_POINTS; i++)
            {
                g.FillEllipse(Brushes.Green, new Rectangle(points[i].X, points[i].Y, POINT_SIZE, POINT_SIZE));
                g.DrawString((i+1).ToString(), font, Brushes.Red, points[i].X + 5, points[i].Y + 5);
            }

            // Dessin des cercles concentriques
            for (int i = 60; i < 2 * WIN_SIZE * 1.42; i += 60)
            {
                g.DrawArc(pen, -i / 2, -i / 2, i, i, 0, 90);
            }
        }

        private void MainForm_Load(object? sender, EventArgs e)
        {
            // Initialise un gnrateur de nombres alatoires
            Random rnd = new Random(DateTime.Now.Millisecond);

            // Instancie tous les PointXY avec des valeurs alatoires pour X et Y
            for (int i = 0; i < NB_POINTS; i++)
            {
                points.Add(new PointXY(rnd.Next(0, WIN_SIZE - (POINT_SIZE + OFFSET)), rnd.Next(0, WIN_SIZE - (POINT_SIZE + OFFSET))));
            }

            // Ajoute des doublons pour le test
            points.Insert(0, points[0]);
            points.Add(points[points.Count() - 1]);

            // Supprimer les doublons en prsrvant les indexes
            List<PointXY>? pts = UniquePreserve(points);

            // Trie tous les PointXY
            pts?.Sort();
        }

        private List<PointXY>? UniquePreserve(List<PointXY> pts)
        {
            if (pts is null)
                return pts;

            HashSet<PointXY> result = new HashSet<PointXY>();
            for (int i = 0; i < pts.Count; i++)
                result.Add(pts[i]);

            return result.ToList();
        }

        private List<PointXY>? Unique(List<PointXY> pts)
        {
            if (pts is null)
                return pts;

            pts.Sort();

            for (int i = 0; i < pts.Count() - 1; i++)
                if (pts[i].Equals(pts[i + 1]))
                    pts.RemoveAt(i);

            return pts;
        }
    }
}