﻿using System;

namespace ComparablePoint
{
    public class PointXY : IComparable, IEquatable<PointXY>
    {
        #region ATTRIBUTS
        private int iX;
        private int iY;
        #endregion

        #region CONSTRUCTEUR
        public PointXY(int x, int y)
        {
            X = x;
            Y = y;
        }
        #endregion

        #region PROPRIETES
        public int X
        {
            get
            {
                return iX;
            }
            set
            {
                if (value >= 0)
                    iX = value;
                else
                    iX = 0;
            }
        }

        public int Y
        {
            get
            {
                return iY;
            }
            set
            {
                if (value >= 0)
                    iY = value;
                else
                    iY = 0;
            }
        }
        #endregion

        #region ICOMPARABLE
        /// <summary>
        /// Permet la comparaison de deux points dans le but de savoir s'ils sont plus loin, 
        /// à même distance, ou plus près de l'origine 0,0.
        /// </summary>
        /// <param name="obj">Objet de comparaison</param>
        /// <returns>-1;0;1</returns>

        public int CompareTo(object? obj)
        {
            PointXY? pt = obj as PointXY;
            if (pt is not null)
            {
                double vecteurThis = Math.Sqrt(this.X * this.X + this.Y * this.Y);
                double vecteurPt = Math.Sqrt(pt.X * pt.X + pt.Y * pt.Y);

                if (vecteurThis > vecteurPt)
                {
                    return 1;
                }

                if (vecteurThis < vecteurPt)
                {
                    return -1;
                }
            }
            return 0;
        }
        #endregion

        #region IEquatable
        public bool Equals(PointXY? other)
        {
            if (other == null)
                return false;

            return this.X == other.X && this.Y == other.Y;
        }
        #endregion

        #region METHODES
        public override string ToString()
        {
            return X.ToString() + ":" + Y.ToString();
        }
        #endregion
    }
}
